%...
%... Burgers equation
%...
%... The MatMol Group (2009)...
%...
%... The well known Burgers equation:
%...
%... z  = -(0.5z^2)  + mu*z   = -z*z  + mu*z   
%...  t            x       xx       x       xx
%...
%...
%... where
%...
%...  z      dependent variable
%...
%...  t       time
%...
%...  x       space
%...
%...  mu      diffusion coefficient
%...
%...
%... an exact solution is given by
%...
%... z(x,t) = burgers_exact(x,t)
%...
%... the boundary conditions (BCs) for Burgers' equation are taken as
%...
%... z(x=x0,t) = burgers_exact(x0,t)     (Dirichlet BCs)
%...
%... z(x=xL,t) = burgers_exact(xL,t)               
%...
%... where
%...
%...  xL      spatial domain
%...
%... the initial condition (IC) for Burgers' equations are taken as
%...
%... z(x,t=0) = burgers_exact(x,0)
%...
%... the following code computes a solution to Burgers' equation
%...
     clear all
%...
%... start a stopwatch timer
     tic
%...
%... set global variables
     global mu
     global x0 xL n Lapl_op Grad_op BM inv_MM G
%...
%... spatial grid
     x0 = 0.0;
     xL = 1.0;
     n  = 101;
     dx = (xL-x0)/(n-1);
     x  = [x0:dx:xL]';
%...
%... model parameter
     mu = 0.001;
%...

%...
%... call to the matfem function for computing the FEM matrices
     [MM, DM, CM, BM,x] = matfem(x, 'dir', 'dir', 'MM', 'DM', 'CM', 'BM', 'quad');
     n			= length(x);     % The new interior points are taken into account
     inv_MM             = inv(MM);
     G                  = zeros(n , 1); % For the boundary conditions
     Grad_op            = inv_MM*CM;   % Gradient operator
     Lapl_op            = -inv_MM*DM;   % Laplacian operator
%... initial conditions
     for i=1:n,
       z(i,1) = burgers_exact(x(i),0);
     end
%...  
%... call to ODE solver 
%...
     t0      = 0;
     dt      = 0.1;
     tfin    = 1;
     tlist   = [t0:dt:tfin];
     steps   = length(tlist)-1;
     zz      = zeros(n , steps+1);
     zz(:,1) = z;
     options = odeset('RelTol',1e-6,'AbsTol',1e-6,'Stats','off');
%...     
for ii = 1 : steps
     [tout, zout] = ode15s('burgers_pde_FEM',[tlist(ii) tlist(ii+1)],z,options);
     z            = zout(end , :)';
     zz(:,ii+1)   = z;
end
     figure
     plot(x,zz,'k');
     xlabel('x');
     ylabel('z(x,t)');
     title('Burgers equation')
     hold on
%...     
     for k=1:steps,
         for i=1:n,
             zexact(i) = burgers_exact(x(i),tlist(k));
         end;
         plot(x,zexact,':k')
     end;
%...
%... read the stopwatch timer
     tcpu=toc;

  
