% The MatMol Group (2009)...
% This is an example including convection and reaction terms
% The system is:
%
%   zt = k*zxx - v*zx + f(z) with f(z) = z - z^3
%   ---> discretization FEM --> Zt = -inv_MM*(k*DM + v*CM + v*BM)*Z + F + inv_MM*G
%
% With boundary conditions
%
%   k*zx(0,t) = v(z(0,t) - zin);           zin = 100*sin(7*t) + 50;
%   zx(L,t) = 0
%
% and initial conditions
%
%   z(x,0) = (-xe.^2 + 2*pi*xe + k*2*pi/v + 50)/150

clear all

% We start by defining the spatial domain and the discretization
L       = pi;                   % Length of the spatial domain
ndisc   = 21;                   % Number of discretization points
n1      = 11;                   % Discretization in the first part of the reactor
n2      = ndisc - n1 + 1;       % Discretization in the second part of the reactor
xe1     = linspace(0,L/3,n1);  	% Spatial discretization in the first part 
xe2     = linspace(L/3,L,n2);	% Spatial discretization in the first part
xe      = [xe1 xe2(2:end)];     % Spatial discretization

% Call to the function which computes the FEM matrices
[MM, DM, CM, BM, xe]= matfem (xe, 'rob', 'neu','MM','DM', 'CM', 'BM', 'quad');
ndisc		    = length(xe);       % The new mesh takes into account the interior points
inv_MM              = inv(MM);          % This will be employed later
G                   = zeros(ndisc , 1); % For the boundary conditions

% System parameters
k   = 0.5;
v   = 0.01;

% With the FEM matrices we compute the espatial operator. 
Sp_oper	= - inv_MM*(k*DM + v*BM + v*CM);

% Initial conditions
z0	=  (-xe.^2 + 2*pi*xe + k*2*pi/v + 50)/150;

% Integration time
tf      = 2;
dt      = 0.02;
tlist	= 0:dt:tf;
steps	= length(tlist)-1;

% Variable initialization (for reservation of memory)
zz      = zeros(ndisc , steps+1);
zz(:,1) = z0;

% Calling the IVP solver
options	= odeset('AbsTol',1e-6,'RelTol',1e-6);
for ii = 1 : steps
 	[t , z]         = ode15s(@ode_ex2, [tlist(ii) tlist(ii+1)], z0, options, Sp_oper, G, inv_MM, v);
	z0              = z(end , :)';	% The initial conditions are renewed
	zz(: , ii+1)	= z0;           % The data are saved
end

% Plot the results
figure
mesh(tlist, xe, zz)
xlabel('Time')
ylabel('Length')
zlabel('Field')