% The MatMol Group (2009)...
%... Algae Bloom (solution using spectral methods with 4 points !)
%...
%... Plankton blooms appear as patches on the surface of water.
%... They can be modeled by (Beltrami, 1997)
%...
%... u  = mu*u   + r*u*(1-u/K)
%...  t       zz
%...
%... where
%...
%...  u      dependent variable
%...
%...  t       time
%...
%...  z       space
%...
%...  mu      diffusion coefficient
%...
%...  r       maximum specific growth rate
%...
%...  K       carrying capacity (logistic growth)
%...
%... the boundary conditions (BCs) model the unfavorable conditions 
%... outside the island-like patch
%...
%... u(z=z0,t) = 0                (Dirichlet BCs)
%...
%... u(z=zL,t) = 0               
%...
%... where
%...
%...  zL      spatial domain
%...
%... there is a minimal size for a patch if the population is not to 
%... die out: zL > pi*sqrt(mu/r). Here, twice this critical size is
%... considered
%...
%... the initial condition (IC) are taken as
%...
%... u(z,t=0) = u_0              (small initial population)
%...
%... Reference
%...
%... E. Beltrami
%... Mathematics for Dynamic Modeling 
%... Academic Press, 1998 (pp 78-84)
%...
%... the following code computes a solution to this problem
%...
      close all
      clear all
%...
%... start a stopwatch timer
     tic
%...
%... set global variables
     global mu r K;
     global z0 zL z n D1;
     global tresh fac vectorized
%...
%... model parameters
     mu = 0.01;
     r = 5;
     K = 1;
%...
%... spatial grid
     z0 = 0.0;
%...
%... select initial diameter of the algae patch (when zL > pi*sqrt(mu/r), the patch
%... will grow, otherwise it will decay)
%...
%      diameter = 'large'
%...
      diameter = 'small'
%...
     switch diameter    
%...
%... the patch is large at the initial time
       case('large')
           zL = 1.5*pi*sqrt(mu/r);
%... the patch is too small
       case('small')
           zL = 0.9*pi*sqrt(mu/r);
%...
     end
%...     
     n = 4;
%...
%... spectral differentiation matrix on a nonuniform grid (stagewise differentiation)
     [D1,z] = chebyshev_spectral_D1(z0,zL,n);
%...
%... initial conditions
     u = 0.1*sin(pi*z/zL);
%...  
%... call to ODE solver (comment/decomment one of the methods to select a solver)
     t0         = 0;
     tf         = 3;
     Dtplot     = 0.3;
     fac        = [];
     tresh      = 1e-12;
     vectorized = 1;
%...
%     method = 'ode15s'     
%...
     method = 'ros23p'     
%...
     switch method    
         %... ode15s
         case('ode15s')
             options = odeset('RelTol',1e-6,'AbsTol',1e-6);
             options = odeset(options,'Vectorized','on');
             options = odeset(options,'Jacobian',@jacobian)
             figure(1);
             colordef white;
             options      = odeset(options,'OutputFcn','odeplot');
             [tout, yout] = ode15s(@algae_bloom_pde,[t0:Dtplot:tf],u,options);
%...
%... ros32p
         case('ros23p')
             hmin      = 0.0001;
             nstepsmax = 1e5;
             abstol    = 1e-6;
             reltol    = 1e-6;
             [tout, yout, eout] = ros23p_solver(@algae_bloom_pde,'Jacobian',t0,tf,u,hmin,nstepsmax,abstol,reltol,Dtplot);
%...     
             figure(3)
             for i=1:length(tout),
                 plot(tout(i),norm(eout(i,:),2),'*r');
                 hold on
             end
             xlabel('t');
             ylabel('e(t)');
%...
     end
%...
%... plot results
     set(0,'defaultaxesfontsize',12,'defaultaxeslinewidth',.7,'defaultlinelinewidth',1.5,'defaultpatchlinewidth',.7);
     colordef white;
     figure(2)
     zint = linspace(z0,zL,100);
     for i = 1:length(tout),
%        yint(i,:)=interp1(z,yout(i,:)',zint,'cubic');
        yint(i,:) = polyval(polyfit(z,yout(i,:)',n),zint);
        plot(zint,yint(i,:),'-g');
        hold on
     end
     xlabel('z');
     ylabel('u');
     title('Algae bloom')
%...
     figure(4)
%     mesh(z,tout,yout);
     surf(z,tout,yout);
     grid off;
%     colormap([0 0 0]);
     axis([z0 zL t0 tf min(min(yout)) max(max(yout))]);
     xlabel('z');
     ylabel('t');
     zlabel('\rho(z,t)');
%...
%... read the stopwatch timer
     tcpu=toc;
  
