%... The MatMol Group (2009)
%...
%... Burgers equation
%...
%... The well known Burgers equation:
%...
%... z  = -(0.5z^2)  + mu*z   = -z*z  + mu*z   
%...  t            x       xx       x       xx
%...
%...
%... where
%...
%...  z      dependent variable
%...
%...  t       time
%...
%...  x       space
%...
%...  mu      diffusion coefficient
%...
%...
%... an exact solution is given by
%...
%... z(x,t) = burgers_exact(x,t)
%...
%... the boundary conditions (BCs) for Burgers' equation are taken as
%...
%... z(x=x0,t) = burgers_exact(x0,t)     (Dirichlet BCs)
%...
%... z(x=xL,t) = burgers_exact(xL,t)               
%...
%... where
%...
%...  xL      spatial domain
%...
%... the initial condition (IC) for Burgers' equations are taken as
%...
%... z(x,t=0) = burgers_exact(x,0)
%...
%... the following code computes a solution to Burgers' equation
%...
     close all
     clear all

%...
%... set global variables
     global mu Phi proj_op lambda
     global x Grad_op

%...
%... start a stopwatch timer
     tic

%...
%... spatial grid
     x0 = 0.0;
     xL = 1.0;
     n  = 201;
     dx = (xL-x0)/(n-1);
     x  = [x0:dx:xL]';
%...
%... model parameter
     mu = 0.001;

%...
%... call to the matfem function for computing the FEM matrices
     [MM, DM, CM, BM]   = matfem(x, 'dir', 'dir', 'MM', 'DM', 'CM', 'BM');
     inv_MM             = inv(MM);
     G                  = zeros(n , 1); % For the boundary conditions
     Grad_op            = inv_MM*CM;   % Gradient operator
     Lapl_op            = -inv_MM*DM;   % Laplacian operator

% Computation of the basis functions
[Phi , lambda]  = matlsd(MM, DM, BM, mu);
neig            = 150;                       % Selection of the number of basis functions
Phi             = Phi(: , 1:neig);
lambda          = lambda(1:neig , 1:neig);
proj_op         = Phi'*MM;

%...
%... initial conditions
     for i=1:n,
       z0(i,1) = burgers_exact(x(i),0);
     end
z_trans = -0.9*x + 1;
z0_bar  = z0 - z_trans;
m0	= proj_op*z0_bar;

%...  
%... call to ODE solver 
%...
     t0      = 0;
     dt      = 0.1;
     tfin    = 1;
     tlist   = [t0:dt:tfin];
     steps   = length(tlist)-1;
     mm      = zeros(neig , steps+1);
     mm(:,1) = m0;
     options = odeset('RelTol',1e-6,'AbsTol',1e-6,'Stats','off');
%...     
for ii = 1 : steps
     [tout, mout] = ode15s('burgers_pde_LSD',[tlist(ii) tlist(ii+1)],m0,options);
     m0           = mout(end , :)';
     mm(:,ii+1)   = m0;
end

% Recovery the transformed field
Z_bar   = Phi*mm;

% Recovery the original field
Z       = Z_bar + repmat(z_trans , 1 , steps+1);


     plot(x,Z,'k');
     xlabel('x');
     ylabel('z(x,t)');
     title('Burgers equation')
     hold on
%...     
     for k=1:steps,
         for i=1:n,
             zexact(i) = burgers_exact(x(i),tlist(k));
         end;
         plot(x,zexact,':k')
     end;
%...
%... read the stopwatch timer
     tcpu=toc;

  
