% The MatMol Group (2009)
%
% This is a simple example in which only diffusion is considered
% The system is:
%
%   zt = k*zxx ---> discretizacion FEM --> MM*Zt = -(k*DM + BM)*Z + G --->
%   This is the same as Zt = -inv_MM*(k*DM + BM)*Z + inv_MM*G
%
% With boundary conditions
%
%   z(0,t) = 3
%   zx(L,t) = 0
%
% and initial conditions
%
%   z(x,0) = -x^2 + 2*pi*x + 3
% 
% The reduced order model is obtained by projection the system equation
% over the basis function. Using the FEM mass matrix for carrying out
% the integrals we obtain:
%
% Phi'*MM*Zt = -(Phi'*MM*inv_MM*(k*DM + BM))*Z + Phi'*MM*inv_MM*G
%
% where Phi are the basis functions. This is equivalent to:
% 
% mt	= -(Phi'*(k*DM + BM)*Phi)*m + Phi'*G = -lambda*m + Phi'*G
%
% where m is the vector of modes of the ROM. The initial conditions are:
%
% m(0)	= Phi'*MM*Z(0);

clear all

% We define now the spatial domain and the discretization
L       = pi;                   % Length of the spatial domain
ndisc   = 21;                   % Number of discretization points
xe      = linspace(0,L,ndisc)'; % Spatial discretization

% Call to the function which computes the FEM matrices
[MM, DM, BM]    = matfem (xe, 'dir', 'neu','MM','DM','BM');
inv_MM          = inv(MM);          % This will be employed later

% System parameters
k       = 0.1;

% Computation of the basis functions
[Phi , lambda]  = matlsd(MM, DM, BM, k);
neig            = 3;    % Selection of the number of basis functions
Phi             = Phi(: , 1:neig);
lambda          = lambda(1:neig , 1:neig);
clear DM BM

% Definition of the transformation for obtaining homogenous boundary condtions
z_transf        = 3*ones(ndisc , 1);

% Initial conditions
z0	= - xe.^2 + 2*pi*xe + 3;
z0_bar  = z0 - z_transf;
m0      = Phi'*MM*z0_bar;

% Integration time
tf      = 150;
dt      = 1;
tlist	= 0:dt:tf;
steps	= length(tlist)-1;

% Variable initialization (for reservation of memory)
mm      = zeros(neig , steps+1);
mm(:,1)	= m0;

% Calling the IVP solver
options	= odeset('AbsTol',1e-6,'RelTol',1e-6);
for ii = 1 : steps
 	[t , m]         = ode15s(@ode_ex1_LSD, [tlist(ii) tlist(ii+1)], m0, options, lambda);
	m0              = m(end , :)';	% The initial conditions are renewed
	mm(: , ii+1)	= m0;           % The data are saved
end

% Recovery the transformed field from the modes and the basis functions
Z_bar   = Phi*mm;

% Recovery the original field
Z       = Z_bar + repmat(z_transf, 1, steps+1);

% Plot the results
figure
mesh(tlist, xe, Z)
xlabel('Time')
ylabel('Length')
zlabel('Field')