% The MatMol Group (2009)
%
% This is an example including convection and reaction terms
% The system is:
%
%   zt = k*zxx - v*zx + f(z) with f(z) = z - z^3
%   ---> discretization FEM --> Zt = -inv_MM*(k*DM + v*CM + v*BM)*Z + F + inv_MM*G
%
% With boundary conditions
%
%   k*zx(0,t) = v(z(0,t) - zin);           zin = 100*sin(7*t) + 50;
%   zx(L,t) = 0
%
% and initial conditions
%
%   z(x,0) = (-xe.^2 + 2*pi*xe + k*2*pi/v + 50)/150
%
% The reduced order model is obtained by projection the system equation
% over the basis function. Using the FEM mass matrix for carrying out
% the integrals we obtain:
%
% Phi'*MM*Zt = -(Phi'*MM*inv_MM*(k*DM + v*CM + v*BM))*Z + Phi'*MM*F +...
%               Phi'*MM*inv_MM*G
%
% where Phi are the basis functions. This is equivalent to:
% 
% mt	= -(Phi'*(k*DM + v*CM + v*BM)*Phi)*m + Phi'*MM*F + Phi'*G =
%       = -(lambda + v*Phi'*BM*Phi)*m + Phi'*MM*F + Phi'*G
%
% where m is the vector of modes of the ROM. The initial conditions are:
%
% m(0)	= Phi'*MM*Z(0);

clear all

% Now we define the spatial domain and the discretization
L       = pi;                   % Length of the spatial domain
ndisc   = 21;                   % Number of discretization points
n1      = 11;                   % Discretization in the first part of the reactor
n2      = ndisc - n1 + 1;       % Discretization in the second part of the reactor
xe1     = linspace(0,L/3,n1);  	% Spatial discretization in the first part 
xe2     = linspace(L/3,L,n2);	% Spatial discretization in the first part
xe      = [xe1 xe2(2:end)]';    % Spatial discretization

% Call to the function which computes the FEM matrices
[MM, DM, CM, BM]    = matfem (xe, 'rob', 'neu','MM','DM', 'CM', 'BM');
inv_MM              = inv(MM);          % This will be employed later


% System parameters
k   = 0.5;
v   = 0.01;

% Computation of the basis functions
[Phi , lambda]  = matlsd(MM, DM, BM, k, v);
neig            = 8;    % Selection of the number of basis functions
Phi             = Phi(: , 1:neig);
lambda          = lambda(1:neig , 1:neig);
clear DM BM

% With the FEM matrices we compute the espatial operator. 
Sp_oper = - (lambda + v*Phi'*CM*Phi);
proj_op = Phi'*MM;

% Integration time
tf      = 2;
dt      = 0.02;
tlist   = 0:dt:tf;
steps   = length(tlist)-1;

% Initial conditions
z0              =  (-xe.^2 + 2*pi*xe + k*2*pi/v + 50)/150;
z_transform     = zeros(steps+1 , 1);
z_transform(1)  = 100*sin(7*tlist(1)) + 50;
z0_bar          = z0 - z_transform(1);
m0              = proj_op*z0_bar;

% Variable initialization (for memory reservation)
mm      = zeros(neig , steps+1);
mm(:,1) = m0;
Z_bar   = zeros(ndisc , steps+1);	Z_bar(:,1)      = z0_bar;
Z       = zeros(ndisc , steps+1);       Z(:,1)          = z0;


% Calling the IVP solver
options	= odeset('AbsTol',1e-6,'RelTol',1e-6);
for ii = 1 : steps
        [t , m]         = ode15s(@ode_ex2_LSD, [tlist(ii) tlist(ii+1)], m0, options, Sp_oper, proj_op, Phi);
        m0              = m(end , :)';	% The initial conditions are renewed
        mm(: , ii+1)    = m0;           % The data are saved

        % Recovery the transformed field
        z_transform(ii+1)       = 100*sin(7*tlist(ii+1)) + 50;
        Z_bar(: , ii+1)         = Phi*m0;
        
        % Recovery the transformed field
        Z(: , ii+1)     = Z_bar(: , ii+1) + z_transform(ii+1);
end


% Plot the results
figure
mesh(tlist, xe, Z)
xlabel('Time')
ylabel('Length')
zlabel('Field')