% The MatMol Group (2009)
%
% This is a simple example in which only diffusion is considered
% The system is:
%
%   zt = k*zxx ---> discretizacion FEM --> MM*Zt = -(k*DM + BM)*Z + G --->
%   This is the same as Zt = -inv_MM*(k*DM + BM)*Z + inv_MM*G
%
% With boundary conditions
%
%   z(0,t) = 3
%   zx(L,t) = 0
%
% and initial conditions
%
%   z(x,0) = -x^2 + 2*pi*x + 3
% 
% The reduced order model is obtained by projection the system equation
% over the basis function. Using the FEM mass matrix for carrying out
% the integrals we obtain:
%
% Phi'*MM*Zt = -(Phi'*MM*inv_MM*(k*DM + BM))*Z + Phi'*MM*inv_MM*G
%
% where Phi are the basis functions. This is equivalent to:
% 
% mt	= -(Phi'*(k*DM + BM)*Phi)*m + Phi'*G
%
% where m is the vector of modes of the ROM. The initial conditions are:
%
% m(0)	= Phi'*MM*Z(0);

clear all

% The first step is to generate the snapshots. In this case this is
% carried out by numerical simulation using Ex1.m. The data were taken
% each 1 unit of time till t= 150 and they were stored in the file 
% "data_simulation_Ex1.mat"
load data_simulation_Ex1

% We define now the spatial domain and the discretization
L       = pi;                   % Length of the spatial domain
ndisc   = 21;                   % Number of discretization points
xe      = linspace(0,L,ndisc)'; % Spatial discretization

% Call to the function which computes the FEM matrices
[MM, DM, BM]    = matfem (xe, 'dir', 'neu','MM','DM','BM');
inv_MM          = inv(MM);          % This will be employed later
G               = zeros(ndisc , 1); % For the boundary conditions
G(1)            = BM(1,1)*3;	    % Boundary condition in the first point

% Computation of the basis functions
[pods]  = matpod(Z , MM , 'd' , 99.999);
Phi     = pods.phi;
neig	= size(Phi , 2);
clear Z

% System parameters
k       = 0.1;

% With the FEM matrices we compute and project the espatial operator.and the
% boundary conditions 
Sp_oper = - Phi'*(k*DM + BM)*Phi;
G	= Phi'*G;
clear DM BM

% Initial conditions
z0	= - xe.^2 + 2*pi*xe + 3;
m0	= Phi'*MM*z0;

% Integration time
tf      = 150;
dt      = 1;
tlist	= 0:dt:tf;
steps	= length(tlist)-1;

% Variable initialization (for reservation of memory)
mm      = zeros(neig , steps+1);
mm(:,1)	= m0;

% Calling the IVP solver
options	= odeset('AbsTol',1e-6,'RelTol',1e-6);
for ii = 1 : steps
 	[t , m]         = ode15s(@ode_ex1_POD, [tlist(ii) tlist(ii+1)], m0, options, Sp_oper, G);
	m0              = m(end , :)';	% The initial conditions are renewed
	mm(: , ii+1)	= m0;           % The data are saved
end

% Recovery the original field from the modes and the basis functions
Z	= Phi*mm;

% Plot the results
figure
mesh(tlist, xe, Z)
xlabel('Time')
ylabel('Length')
zlabel('Field')