% The MatMol Group (2009)
%
% This is an example including convection and reaction terms
% The system is:
%
%   zt = k*zxx - v*zx + f(z) with f(z) = z - z^3
%   ---> discretization FEM --> Zt = -inv_MM*(k*DM + v*CM + v*BM)*Z + F + inv_MM*G
%
% With boundary conditions
%
%   k*zx(0,t) = v(z(0,t) - zin);           zin = 100*sin(7*t) + 50;
%   zx(L,t) = 0
%
% and initial conditions
%
%   z(x,0) = (-xe.^2 + 2*pi*xe + k*2*pi/v + 50)/150
%
% The reduced order model is obtained by projection the system equation
% over the basis function. Using the FEM mass matrix for carrying out
% the integrals we obtain:
%
% Phi'*MM*Zt = -(Phi'*MM*inv_MM*(k*DM + v*CM + v*BM))*Z + Phi'*MM*F +...
%               Phi'*MM*inv_MM*G
%
% where Phi are the basis functions. This is equivalent to:
% 
% mt	= -(Phi'*(k*DM + v*CM + v*BM)*Phi)*m + Phi'*MM*F + Phi'*G
%
% where m is the vector of modes of the ROM. The initial conditions are:
%
% m(0)	= Phi'*MM*Z(0);

clear all

% The first step is to generate the snapshots. In this case this is
% carried out by numerical simulation using Ex1.m. The data were taken
% each 0.01 unit of time till t= 1.2 and they were stored in the file 
% "data_simulation_Ex2.mat"
load data_simulation_Ex2

% Now we define the spatial domain and the discretization
L       = pi;                   % Length of the spatial domain
ndisc   = 21;                   % Number of discretization points
n1      = 11;                   % Discretization in the first part of the reactor
n2      = ndisc - n1 + 1;       % Discretization in the second part of the reactor
xe1     = linspace(0,L/3,n1);  	% Spatial discretization in the first part 
xe2     = linspace(L/3,L,n2);	% Spatial discretization in the first part
xe      = [xe1 xe2(2:end)]';    % Spatial discretization

% Call to the function which computes the FEM matrices
[MM, DM, CM, BM]    = matfem (xe, 'rob', 'neu','MM','DM', 'CM', 'BM');
inv_MM              = inv(MM);          % This will be employed later
G                   = zeros(ndisc , 1); % For the boundary conditions

% Computation of the basis functions
[pods] 	= matpod(Z , MM , 'd' , 99.999);
Phi	= pods.phi;
neig	= size(Phi , 2);
proj_op	= Phi'*MM; 	% Projection operator
clear Z

% System parameters
k   = 0.5;
v   = 0.01;

% With the FEM matrices we compute the espatial operator. 
Sp_oper	= - Phi'*(k*DM + v*BM + v*CM)*Phi;

% Initial conditions
z0	=  (-xe.^2 + 2*pi*xe + k*2*pi/v + 50)/150;
m0	= proj_op*z0;

% Integration time
tf      = 2;
dt      = 0.02;
tlist	= 0:dt:tf;
steps	= length(tlist)-1;

% Variable initialization (for reservation of memory)
mm      = zeros(neig , steps+1);
mm(:,1)	= m0;

% Calling the IVP solver
options	= odeset('AbsTol',1e-6,'RelTol',1e-6);
for ii = 1 : steps
 	[t , m]         = ode15s(@ode_ex2_POD, [tlist(ii) tlist(ii+1)], m0, options, Sp_oper, G, proj_op, v, Phi);
	m0              = m(end , :)';	% The initial conditions are renewed
	mm(: , ii+1)	= m0;           % The data are saved
end

% Recovery the original field
Z	= Phi*mm;

% Plot the results
figure
mesh(tlist, xe, Z)
xlabel('Time')
ylabel('Length')
zlabel('Field')